﻿using ChromeStarter.Models;
using Microsoft.Win32;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace ChromeStarter
{
    /// <summary>
    /// MainWindow.xaml에 대한 상호 작용 논리
    /// </summary>
    public partial class MainWindow : Window
    {
        private readonly string PROFILE_PATTERN = "Profile ";
        private readonly string CHROME_PATH_KEY = "ChromePath";

        private string _chromePath;
        string _savePath;
        private ProfilePlanCollection _profilePlanCollection;

        public MainWindow()
        {
            InitializeComponent();
            InitInstance();
        }

        private void InitInstance()
        {
            _chromePath = ConfigurationManager.AppSettings[CHROME_PATH_KEY];
            _savePath = $@"{AppDomain.CurrentDomain.BaseDirectory}\cache.json";
            _profilePlanCollection = this.DataContext as ProfilePlanCollection;

            if (!string.IsNullOrEmpty(_chromePath) )
                tbChromePath.Text = _chromePath;

            if (_profilePlanCollection == null)
            {
                ShowErrorMessage("윈도우 컨텍스트를 확인할 수 없습니다.");
                return;
            }

            string localChromeUserDataPath = $@"{Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData)}\Google\Chrome\User Data";
            if (!Directory.Exists(localChromeUserDataPath))
            {
                ShowErrorMessage("사용자 로컬 폴더를 찾을 수 없습니다. 크롬이 설치되어있는지 확인하십시오.");
                return;
            }

            List<string> profiles = GetProfiles(localChromeUserDataPath);
            if (profiles == null || profiles.Count < 1)
            {
                ShowErrorMessage("사용자 프로필을 확인할 수 없습니다. 크롬에서 사용자 프로필을 생성하십시오.");
                return;
            }

            Dictionary<string, ProfilePlan> cache = new Dictionary<string, ProfilePlan>();
            if (File.Exists(_savePath))
            {
                string jsonString = File.ReadAllText(_savePath);
                cache = JsonConvert.DeserializeObject<Dictionary<string, ProfilePlan>>(jsonString);
            }

            foreach (string profile in profiles)
            {
                if (cache.ContainsKey(profile))
                    _profilePlanCollection.ProfilePlans.Add(new ProfilePlan() { Name = profile, StartPage = cache[profile].StartPage });
                else
                    _profilePlanCollection.ProfilePlans.Add(new ProfilePlan() { Name = profile });
            }
        }

        private List<string> GetProfiles(string localChromeUserDataPath)
        {
            string[] subDirectories = Directory.GetDirectories(localChromeUserDataPath);
            var filteredDirectories = subDirectories.Where(dir => System.IO.Path.GetFileName(dir).StartsWith(PROFILE_PATTERN));
            if (!filteredDirectories.Any())
                return null;

            List<string> profiles = new List<string>();
            foreach (string dir in filteredDirectories)
            {
                profiles.Add(System.IO.Path.GetFileName(dir));
            }

            return profiles;
        }

        private void ShowErrorMessage(string message)
        {
            MessageBox.Show(message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
        }

        private void SavePlanCache(ProfilePlanCollection profilePlanCollection)
        {
            Dictionary<string, ProfilePlan> cache = new Dictionary<string, ProfilePlan>();
            foreach (var profilePlan in profilePlanCollection.ProfilePlans)
            {
                cache.Add(profilePlan.Name, profilePlan);
            }

            string jsonString = JsonConvert.SerializeObject(cache);

           
            File.WriteAllText(_savePath, jsonString);

        }

        private void btnFindChromePath_Click(object sender, RoutedEventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            if (dlg.ShowDialog() != true)
                return;

            tbChromePath.Text = dlg.FileName;
            Configuration config = ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None);
            config.AppSettings.Settings[CHROME_PATH_KEY].Value = dlg.FileName;

            config.Save(ConfigurationSaveMode.Modified);
        }

        private void btnStart_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (_profilePlanCollection == null || _profilePlanCollection.GetCommandParameter().Count < 1)
                {
                    ShowErrorMessage("사용자 프로필 계획을 생성할 수 없습니다.");
                    return;
                }

                foreach (string profilePlan in _profilePlanCollection.GetCommandParameter())
                {
                    Process.Start(tbChromePath.Text, profilePlan);
                }

                SavePlanCache(_profilePlanCollection);
            }
            catch (Exception ex)
            {
                ShowErrorMessage(ex.Message);
            }
        }
    }
}
